#ifndef _CHRONOPTICS_TOF_CSF_CAMERA_H_
#define _CHRONOPTICS_TOF_CSF_CAMERA_H_

#include <chronoptics/tof/camera.h>
#include <chronoptics/tof/processing_config.h>
#include <chronoptics/tof/data.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The csf camera class contains all the logic to generate a stream of depth
 * data from a csf file
 */
typedef struct tof_csf_camera* tof_csf_camera_t;

/** Destruct tof_csf_camera */
TOF_EXPORT void tof_csf_camera_delete(tof_csf_camera_t ptr);

/** Construct a csf camera from a csf file
 * @param ptr Pointer to class
 * @param processing_config A processing config
 * @param file_location File location of the csf file
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_csf_camera_t tof_csf_camera_new(const tof_processing_config_t processing_config, const char* file_location, tof_error_t *error);

/** Construct a csf camera from a csf file with a processing config generated by
 * default_processing from the camera config
 * @param ptr Pointer to class
 * @param file_location File location of the csf file
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_csf_camera_t tof_csf_camera_new_simple(const char* file_location, tof_error_t *error);

/** Get frames at the specified position
 * @param ptr Pointer to class
 * @param frame_nr Which set of frames to process
 * @param frames Data frames
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_csf_camera_get_frames_at(tof_csf_camera_t ptr, size_t frame_nr, tof_data_t* frames, size_t capacity, tof_error_t *error);

/** Get the location (count) of the current frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The currenc frame location
 */
TOF_EXPORT size_t tof_csf_camera_get_current_frame(const tof_csf_camera_t ptr, tof_error_t *error);

/** Get the amount of frame lists this csf file can open/generate
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The number of frames in the csf file
 */
TOF_EXPORT size_t tof_csf_camera_get_frame_number(const tof_csf_camera_t ptr, tof_error_t *error);

/** Get the FPS If the FPS is greater than 0 the time between subsequent
 * function calls to get_frames or get_frames_at will be limited to the FPS This
 * is done by sleeping on the next call to get_frames() if the time between
 * calls is smaller than the time duration dictated by the fps.

 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The frames per second
 */
TOF_EXPORT size_t tof_csf_camera_get_fps(const tof_csf_camera_t ptr, tof_error_t *error);

/** Set the FPS If the FPS is greater than 0 the time between subsequent
 * function calls to get_frames or get_frames_at will be limited to the FPS This
 * is done by sleeping on the next call to get_frames() if the time between
 * calls is smaller than the time duration dictated by the fps.

 * @param ptr Pointer to class
 * @param fps The frames per second
 * @param error Pointer to error
 */
TOF_EXPORT void tof_csf_camera_set_fps(tof_csf_camera_t ptr, size_t fps, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
