import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera_config import *
from .calibration import *
from .data import *


class DepthStreamType:
    """
    The possible depth streams that can be streamed from the camera
    """

    RAW = 0
    "Stream raw data"

    RADIAL_INTENSITY = 1
    "Stream both radial and intensity"

    RADIAL = 2
    "Stream radial"

    INTENSITY = 3
    "Stream intensity"



class DiscoveryMessage():
    """
    A gige discovery message containing camera information
    """
    def __del__(self):
        if self._ptr:
            lib.tof_discovery_message_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new discovery_message class
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def ip(self) -> str:
        """
        Get the camera's ip address

        Returns
        ----------
        str
            Ip address
        """
        return lib.tof_discovery_message_ip(self._ptr, get_error()).decode("utf-8")


    def serial(self) -> str:
        """
        Get the camera's serial

        Returns
        ----------
        str
            Serial
        """
        return lib.tof_discovery_message_serial(self._ptr, get_error()).decode("utf-8")


class PersistentIp():
    """
    A persistent ip data structure
    """
    def __del__(self):
        if self._ptr:
            lib.tof_persistent_ip_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ip: Optional[str] = None, netmask: Optional[str] = None, gateway: Optional[str] = None, enabled: Optional[bool] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new persistent_ip class
        ----------

        Create persistent ip

        Parameters
        ----------
        ip : str
                IP address
        netmask : str
                Netmask
        gateway : str
                Gateway
        enabled : bool
                Enable or disable persistent ip
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif ip != None and netmask != None and gateway != None and enabled != None:
            self._ptr = lib.tof_persistent_ip_new(ip.encode("utf-8"), netmask.encode("utf-8"), gateway.encode("utf-8"), enabled, get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def getIp(self) -> str:
        """
        Get the IP address

        Returns
        ----------
        str
            the IP address
        """
        return lib.tof_persistent_ip_get_ip(self._ptr, get_error()).decode("utf-8")


    def setIp(self, ip: str) -> None:
        """
        Set the IP address

        Parameters
        ----------
        ip : str
            the IP address
        """
        return lib.tof_persistent_ip_set_ip(self._ptr, ip.encode("utf-8"), get_error())


    def getNetmask(self) -> str:
        """
        Get the netmask

        Returns
        ----------
        str
            The netmask
        """
        return lib.tof_persistent_ip_get_netmask(self._ptr, get_error()).decode("utf-8")


    def setNetmask(self, netmask: str) -> None:
        """
        Set the netmask

        Parameters
        ----------
        netmask : str
            The netmask
        """
        return lib.tof_persistent_ip_set_netmask(self._ptr, netmask.encode("utf-8"), get_error())


    def getGateway(self) -> str:
        """
        Get the gateway

        Returns
        ----------
        str
            The gateway
        """
        return lib.tof_persistent_ip_get_gateway(self._ptr, get_error()).decode("utf-8")


    def setGateway(self, gateway: str) -> None:
        """
        Set the gateway

        Parameters
        ----------
        gateway : str
            The gateway
        """
        return lib.tof_persistent_ip_set_gateway(self._ptr, gateway.encode("utf-8"), get_error())


    def getEnabled(self) -> bool:
        """
        Get if persistent ip is enabled

        Returns
        ----------
        bool
            Is enabled
        """
        return lib.tof_persistent_ip_get_enabled(self._ptr, get_error())


    def setEnabled(self, enabled: bool) -> None:
        """
        Set if persistent ip is enabled

        Parameters
        ----------
        enabled : bool
            Is enabled
        """
        return lib.tof_persistent_ip_set_enabled(self._ptr, enabled, get_error())


class GigeInterface():
    """
    Connect to a camera directly using the gige interface
    """
    def __del__(self):
        if self._ptr:
            lib.tof_gige_interface_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, port: Optional[int] = 0, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new gige_interface class
        ----------

        Create an instance of the gige interface

        Parameters
        ----------
        port : int
                The port to use for GVCP communication, if 0 is specified it will
                use an OS assigned port
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif port != None:
            self._ptr = lib.tof_gige_interface_new(port, get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def discover(self) -> List[DiscoveryMessage]:
        """
        Detect all the cameras on the network

        Returns
        ----------
        List[DiscoveryMessage]
            A list of discovery messages
        """
        size = lib.tof_gige_interface_discover(self._ptr, ctypes.c_void_p(), 0, get_error())
        array = (ctypes.c_void_p * size)()
        lib.tof_gige_interface_discover(self._ptr, array, size, get_error())
        return [DiscoveryMessage(ptr = item) for item in array]

    def discoverOne(self) -> DiscoveryMessage:
        """
        Returns a message of the first camera that responded

        Returns
        ----------
        DiscoveryMessage
            A discovery message
        """
        return DiscoveryMessage(ptr = lib.tof_gige_interface_discover_one(self._ptr, get_error()))


    def find(self, serial: str) -> DiscoveryMessage:
        """
        Detect a specific camera on the network, will throw an error if it can't
        find the camera

        Parameters
        ----------
        serial : str
            Serial number of camera to find

        Returns
        ----------
        DiscoveryMessage
            A discovery message
        """
        return DiscoveryMessage(ptr = lib.tof_gige_interface_find(self._ptr, serial.encode("utf-8"), get_error()))


    def connect(self, message: DiscoveryMessage) -> None:
        """
        Connect to the camera and gain exclusive access

        Parameters
        ----------
        message : DiscoveryMessage
            Discovery message of the camera to connect to
        """
        return lib.tof_gige_interface_connect(self._ptr, message._ptr, get_error())


    def disconnect(self) -> None:
        """
        Disconnect from the camera
        """
        return lib.tof_gige_interface_disconnect(self._ptr, get_error())


    def isConnected(self) -> bool:
        """
        Check whether a connection is still maintained with the camera

        Returns
        ----------
        bool
            Is connected
        """
        return lib.tof_gige_interface_is_connected(self._ptr, get_error())


    def getStreamPacketSize(self) -> int:
        """
        Get stream packet size

        Returns
        ----------
        int
            The packet size
        """
        return lib.tof_gige_interface_get_stream_packet_size(self._ptr, get_error())


    def setStreamPacketSize(self, packet_size: int) -> None:
        """
        Set stream packet size

        Parameters
        ----------
        packet_size : int
            The packet size
        """
        return lib.tof_gige_interface_set_stream_packet_size(self._ptr, packet_size, get_error())


    def testStreamPacketSize(self, port: int = 0) -> bool:
        """
        Test whether using the set packet size works

        Parameters
        ----------
        port : int
            Port to send test packets to

        Returns
        ----------
        bool
            Packet size works
        """
        return lib.tof_gige_interface_test_stream_packet_size(self._ptr, port, get_error())


    def getDelay(self) -> int:
        """
        Get delay between network packets

        Returns
        ----------
        int
            The delay, depending on the camera software it is in nanoseconds or in
            cpu instruction
        """
        return lib.tof_gige_interface_get_delay(self._ptr, get_error())


    def setDelay(self, delay: int) -> None:
        """
        Set delay between network packets

        Parameters
        ----------
        delay : int
            The delay, depending on the camera software it is in nanoseconds or in
            cpu instruction
        """
        return lib.tof_gige_interface_set_delay(self._ptr, delay, get_error())


    def downloadConfiguration(self, port: int = 0) -> CameraConfig:
        """
        Download camera configuration from the camera

        Parameters
        ----------
        port : int
            Port to download configuration on

        Returns
        ----------
        CameraConfig
            Camera configuration
        """
        return CameraConfig(ptr = lib.tof_gige_interface_download_configuration(self._ptr, port, get_error()))


    def downloadCalibration(self, port: int = 0) -> Calibration:
        """
        Download calibration from the camera

        Parameters
        ----------
        port : int
            Port to download configuration on

        Returns
        ----------
        Calibration
            Calibration
        """
        return Calibration(ptr = lib.tof_gige_interface_download_calibration(self._ptr, port, get_error()))


    def uploadConfiguration(self, config: CameraConfig, port: int = 0) -> None:
        """
        Upload camera configuration to the camera

        Parameters
        ----------
        config : CameraConfig
            Camera configuration
        port : int
            Port to download configuration on
        """
        return lib.tof_gige_interface_upload_configuration(self._ptr, config._ptr, port, get_error())


    def depthStreamCapable(self, stream: DepthStreamType) -> bool:
        """
        Check whether this camera is capable of streaming the specified type

        Parameters
        ----------
        stream : DepthStreamType
            Depth stream type

        Returns
        ----------
        bool
            Is capable
        """
        return lib.tof_gige_interface_depth_stream_capable(self._ptr, stream, get_error())


    def startDepthStream(self, stream: DepthStreamType, port: int = 0) -> None:
        """
        Start the depth stream

        Parameters
        ----------
        stream : DepthStreamType
            Depth stream type
        port : int
            Port to receive depth data on
        """
        return lib.tof_gige_interface_start_depth_stream(self._ptr, stream, port, get_error())


    def stopDepthStream(self) -> None:
        """
        Stop the depth stream
        """
        return lib.tof_gige_interface_stop_depth_stream(self._ptr, get_error())


    def hasDepthData(self) -> bool:
        """
        Returns true if a depth frame is available

        Returns
        ----------
        bool
            Depth frame available
        """
        return lib.tof_gige_interface_has_depth_data(self._ptr, get_error())


    def getDepthData(self) -> Data:
        """
        Get a depth frame

        Returns
        ----------
        Data
            Depth frame
        """
        return Data(ptr = lib.tof_gige_interface_get_depth_data(self._ptr, get_error()))


    def imageStreamCapable(self) -> bool:
        """
        Check whether camera supports an image stream

        Returns
        ----------
        bool
            Camera supports an image stream
        """
        return lib.tof_gige_interface_image_stream_capable(self._ptr, get_error())


    def startImageStream(self, port: int = 0) -> None:
        """
        Start image stream

        Parameters
        ----------
        port : int
            Port to receive image data on
        """
        return lib.tof_gige_interface_start_image_stream(self._ptr, port, get_error())


    def stopImageStream(self) -> None:
        """
        Stop image stream
        """
        return lib.tof_gige_interface_stop_image_stream(self._ptr, get_error())


    def hasImageData(self) -> bool:
        """
        Check whether image frame is available

        Returns
        ----------
        bool
            Image frame available
        """
        return lib.tof_gige_interface_has_image_data(self._ptr, get_error())


    def getImageData(self) -> Data:
        """
        Get image frame

        Returns
        ----------
        Data
            Image data
        """
        return Data(ptr = lib.tof_gige_interface_get_image_data(self._ptr, get_error()))


    def persistentIpCapable(self) -> bool:
        """
        Get whether the camera is persistent ip capable

        Returns
        ----------
        bool
            Persistent IP capable
        """
        return lib.tof_gige_interface_persistent_ip_capable(self._ptr, get_error())


    def getPersistentIp(self) -> PersistentIp:
        """
        Get the current persistent ip settings

        Returns
        ----------
        PersistentIp
            Persistent IP
        """
        return PersistentIp(ptr = lib.tof_gige_interface_get_persistent_ip(self._ptr, get_error()))


    def setPersistentIp(self, persistent_ip: PersistentIp) -> None:
        """
        Set the persistent ip settings. The persistent ip settings will be active on
        next reboot.

        Parameters
        ----------
        persistent_ip : PersistentIp
            The persistent ip
        """
        return lib.tof_gige_interface_set_persistent_ip(self._ptr, persistent_ip._ptr, get_error())


    def getUserPointerCapacity(self) -> int:
        """
        Get the capacity for amount of user pointers that can be registered

        Returns
        ----------
        int
            User pointer capacity
        """
        return lib.tof_gige_interface_get_user_pointer_capacity(self._ptr, get_error())


    def setUserPointerCapacity(self, capacity: int) -> None:
        """
        Set the capacity for amount of user pointers that can be registered

        Parameters
        ----------
        capacity : int
            User pointer capacity
        """
        return lib.tof_gige_interface_set_user_pointer_capacity(self._ptr, capacity, get_error())


    def version(self) -> str:
        """
        Get the tof library version running on the camera

        Returns
        ----------
        str
            Version
        """
        return lib.tof_gige_interface_version(self._ptr, get_error()).decode("utf-8")


    def softwareTriggerCapable(self) -> bool:
        """
        Check whether the camera is capable of software trigger

        Returns
        ----------
        bool
            Software trigger capable
        """
        return lib.tof_gige_interface_software_trigger_capable(self._ptr, get_error())


    def softwareTrigger(self) -> None:
        """
        Software trigger the camera
        """
        return lib.tof_gige_interface_software_trigger(self._ptr, get_error())


